local BaikCore = LibStub("AceAddon-3.0"):GetAddon("BaikCore")
local Assert = BaikCore.Assert
local Hook = BaikCore.Hook

-- Add Module
local Ui = {}
BaikCore.Ui = Ui

-- Private Variables
local HiddenFrame = CreateFrame("Frame", "BaikCore_HiddenFrame", UIParent)
HiddenFrame:Hide()
local SetPointNil = function(...) end

-- Ui API
function Ui:CreateColor(r, g, b, a)
    Assert:Number(r)
    Assert:Number(g)
    Assert:Number(b)
    Assert:Number(a)

    local color = {}

    color.r = r
    color.g = g
    color.b = b
    color.a = a

    return color
end

function Ui:SetPointFinal(fromFrame, fromPt, toFrame, toPt, x, y, clear)
    if clear then
        fromFrame:ClearAllPoints()
    end

    if not fromFrame.SetPointCached then
        fromFrame.SetPointCached = fromFrame.SetPoint
        fromFrame.SetPoint = SetPointNil
        fromFrame.ignoreFramePositionManager = true
    end

    fromFrame:SetPointCached(fromPt, toFrame, toPt, x, y)
end

function Ui:UndoSetPointFinal(fromFrame)
    if not fromFrame.SetPointCached then
        return
    end

    fromFrame.SetPoint = fromFrame.SetPointCached
    fromFrame.SetPointCached = nil
end

function Ui:HideFinal(frame)
    local parent = frame:GetParent()

    if frame.cachedParent == nil then
        frame.cachedParent = parent
        frame.ignoreFramePositionManager = true
    end

    frame:SetParent(HiddenFrame)
end

function Ui:UndoHideFinal(frame)
    local parent = frame.cachedParent

    if not parent then
        return
    end

    frame:SetParent(parent)
    frame.cachedParent = nil
end

function Ui:SetSizeFinal(hookMod, frame, width, height)
    if not width then
        frame:SetWidth(width)
    elseif not height then
        frame:SetHeight(height)
    else
        frame:SetSize(width, height)
    end

    Hook:EnsureNoOp(hookMod, frame, "SetWidth")
    Hook:EnsureNoOp(hookMod, frame, "SetHeight")
    Hook:EnsureNoOp(hookMod, frame, "SetSize")
end

function Ui:GetClassColor(unit)
    Assert:String(unit)

    local class = select(2, UnitClass(unit))

    return RAID_CLASS_COLORS[class]
end

-- The following function is created by Daniel Stephens
-- texture			- Texture
-- canvasFrame      - Canvas Frame (for anchoring)
-- startX,startY    - Coordinate of start of line
-- endX,endY		- Coordinate of end of line
-- lineWidth        - Width of line
-- relPoint			- Relative point on canvas to interpret coords (Default BOTTOMLEFT)
function Ui:DrawLine(texture, canvasFrame, startX, startY, endX, endY, lineWidth, relPoint)
	if (not relPoint) then relPoint = "BOTTOMLEFT"; end

	-- Determine dimensions and center point of line
	local dx,dy = endX - startX, endY - startY;
	local cx,cy = (startX + endX) / 2, (startY + endY) / 2;

	-- Normalize direction if necessary
	if (dx < 0) then
		dx,dy = -dx,-dy;
	end

	-- Calculate actual length of line
	local lineLength = sqrt((dx * dx) + (dy * dy));

	-- Quick escape if it'sin zero length
	if (lineLength == 0) then
		texture:SetTexCoord(0,0,0,0,0,0,0,0);
		texture:SetPoint("BOTTOMLEFT", canvasFrame, relPoint, cx,cy);
		texture:SetPoint("TOPRIGHT",   canvasFrame, relPoint, cx,cy);
		return;
	end

	-- Sin and Cosine of rotation, and combination (for later)
	local sin, cos = -dy / lineLength, dx / lineLength;
	local sinCos = sin * cos;

	-- Calculate bounding box size and texture coordinates
	local boundingWidth, boundingHeight, bottomLeftX, bottomLeftY, topLeftX, topLeftY, topRightX, topRightY, bottomRightX, bottomRightY;
	if (dy >= 0) then
		boundingWidth = ((lineLength * cos) - (lineWidth * sin));
		boundingHeight = ((lineWidth * cos) - (lineLength * sin));

		bottomLeftX = (lineWidth / lineLength) * sinCos;
		bottomLeftY = sin * sin;
		bottomRightY = (lineLength / lineWidth) * sinCos;
		bottomRightX = 1 - bottomLeftY;

		topLeftX = bottomLeftY;
		topLeftY = 1 - bottomRightY;
		topRightX = 1 - bottomLeftX;
		topRightY = bottomRightX;
	else
		boundingWidth = ((lineLength * cos) + (lineWidth * sin));
		boundingHeight = ((lineWidth * cos) + (lineLength * sin));

		bottomLeftX = sin * sin;
		bottomLeftY = -(lineLength / lineWidth) * sinCos;
		bottomRightX = 1 + (lineWidth / lineLength) * sinCos;
		bottomRightY = bottomLeftX;

		topLeftX = 1 - bottomRightX;
		topLeftY = 1 - bottomLeftX;
		topRightY = 1 - bottomLeftY;
		topRightX = topLeftY;
	end

	-- Set texture coordinates and anchors
	texture:ClearAllPoints();
	texture:SetTexCoord(topLeftX, topLeftY, bottomLeftX, bottomLeftY, topRightX, topRightY, bottomRightX, bottomRightY);
	texture:SetPoint("BOTTOMLEFT", canvasFrame, relPoint, cx - boundingWidth, cy - boundingHeight);
	texture:SetPoint("TOPRIGHT",   canvasFrame, relPoint, cx + boundingWidth, cy + boundingHeight);
end
